package com.ibm.commerce.applepay.order.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.ArrayList;
import java.util.Map;

import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.member.constants.ECMemberConstants;
import com.ibm.commerce.order.utils.OrderConstants;
import com.ibm.commerce.orderitems.commands.OrderItemUpdateCmd;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;

/**
 * This is the default implementation of the controller command <code>ApplePayOrderUpdateCmd</code>.
 * It processes the apple pay order update logic, including payment data update, shipping contact 
 * and shipping method update.
 * 
 * 
 *<h4>Behaviour:</h4>
 * <ul>
 * 		<li>If AddressData is not empty, then check if the address is existed or not. 
 * 			If yes, return the addressId.
 * 			If no, create a new address and return the address Id.
 * 			Set the addressId to the requestProperties.
 * 		</li> 
	    <li>Call OrderItemUpdateCmd to update the apple pay order. </li> 		
	</ul>	

 */
public class ApplePayOrderUpdateCmdImpl extends ApplePayOrderBaseCmdImpl implements
		ApplePayOrderUpdateCmd {

	
	 /** The IBM Copyright notice field. */
    public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

    /** The name of this class for trace/logging/debuging purposes. */
    private static final String CLASS_NAME = "com.ibm.commerce.applepay.order.commands.ApplePayOrderUpdateCmdImpl";
    
    private static final String APPLE_PAY_ADDRESS="applePayAddress";
    
    
    private String orderId = null;
    
    private String shipModeId = null;
    
    private Map<String, String> addressMap = null;
    
    

	/**
	 * Set order id.
	 */
	public void setOrderId(String iorderId) {
		orderId = iorderId;
	}

	/**
	 * Get order id.
	 */
	public String getOrderId() {
		return orderId;
	}

	/**
	 * Set ship mode id.
	 */
	public void setShipModeId(String ishipmodeId) {
		shipModeId = ishipmodeId;
	}

	/**
	 * Get ship mode id.
	 */
	public String getShipModeId() {
		return shipModeId;
	}
    
	/**
	 * The WebController calls the setRequestProperties method before invoking the execute method in this command.
	 * @param aReqParms request properties.
	 * @exception ECException
	 */
	public void setRequestProperties(TypedProperty aRequestProperties) throws ECException {
        final String methodName = "setRequestProperties";
        ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
        if (ECTrace.traceEnabled(ECTraceIdentifiers.COMPONENT_ORDER)) {
            ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName,
                "The request properties:"
                    + (aRequestProperties == null ? "null" : aRequestProperties.toString()));
        }
        requestProperties = aRequestProperties;
        if (requestProperties.containsKey(APPLE_PAY_ADDRESS)){
			addressMap = (Map<String, String>) requestProperties.get(APPLE_PAY_ADDRESS);
			requestProperties.remove(APPLE_PAY_ADDRESS);
		}
        
        if (requestProperties.containsKey(OrderConstants.EC_SHIPMODE_ID)){
        	shipModeId = requestProperties.getString(OrderConstants.EC_SHIPMODE_ID, null);
        	System.out.println("shipModeId="+shipModeId);
        }
     
        ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
        
	}
    
	/**
	 * PerformExecute logic.
	 */
	public void performExecute() throws ECException {
		final String strMethod = "performExecute";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		String addressId = null;
		if (addressMap != null  && !addressMap.isEmpty()){
			setAddressMap(addressMap);
			ArrayList<String> propertyKey = new ArrayList<String>(){
				{
		    	 add(ECMemberConstants.EC_ADDR_CITY);
		    	 add(ECMemberConstants.EC_ADDR_COUNTRY);
		    	 add(ECMemberConstants.EC_ADDR_STATE);
		    	 add(ECMemberConstants.EC_ADDR_ZIPCODE);
				}
			};
			setToTypedPropertyKey(propertyKey);
			addressId = queryAddOrUpdateAddress();
			if (addressId !=null){
				requestProperties.put(OrderConstants.EC_ADDRESS_ID, addressId);
				
			}else{
				//throw exception
			}
			
		}
		
		if (shipModeId != null){
			requestProperties.put(OrderConstants.EC_SHIPMODE_ID, shipModeId);
		}
		
		OrderItemUpdateCmd orderItemUpdateCmd = getOrderItemUpdateCmd();			  
		CommandContext newCommandContext = (CommandContext) commandContext.clone();
		newCommandContext.setRequestProperties(requestProperties);			  
		orderItemUpdateCmd.setCommandContext(newCommandContext);			  
		orderItemUpdateCmd.setAccCheck(false);
		orderItemUpdateCmd.setRequestProperties(requestProperties);
		orderItemUpdateCmd.execute();
		responseProperties = orderItemUpdateCmd.getResponseProperties();
		
		if (addressId != null){
			responseProperties.put(OrderConstants.EC_ADDRESS_ID, addressId);
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
	}
	
	
	/**
	 * Get the {@link OrderItemUpdateCmd} .
	 * 
	 * @throws ECException
	 * @return The command.
	 */
	protected OrderItemUpdateCmd getOrderItemUpdateCmd() throws ECException {
		return (OrderItemUpdateCmd)CommandFactory.createCommand(OrderItemUpdateCmd.NAME, commandContext.getStoreId());
	}
	
}
