package com.ibm.commerce.applepay.payment.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.json.JSONException;
import org.apache.commons.json.JSONObject;

import com.ibm.commerce.command.ControllerCommandImpl;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECMessageHelper;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;

/**
 * This class is the default implementation of <code>ApplePayMerchantValidationCmd</code>
 * It is used for performing the merchant validation step for ApplePay on Web.
 * 
 * Input: the merchant validation URL which is retrieved from browser.
 * Output: the merchant session object which is returned from ApplePay merchant validation service.
 *
 */
public class ApplePayMerchantValidationCmdImpl extends ControllerCommandImpl implements
		ApplePayMerchantValidationCmd {

	public static final String CLASS_NAME = ApplePayMerchantValidationCmdImpl.class.getName();
	private static final Long TRACE_COMPONENT = ECTraceIdentifiers.COMPONENT_PPC;
	
	private String validationURL = "";
	private String session = "";
	private String paymentSystem = null;
	private String paymentConfigGroup = null;
	
	public static final String MERCHANT_IDENTIFIER = "merchantIdentifier";
	public static final String DOMAIN_NAME = "domainName";
	public static final String DISPLAY_NAME = "displayName";
	public static final String APPLE_PAY_PLUGIN_NAME = "ApplePay";
	public static final String VALIDATION_URL = "validationURL";
	public static final String MERCHANT_SESSION = "merchantSession";
	public static final String MERCHANT_SESSION_IDENTIFIER = "merchantSessionIdentifier";
	public static final String PAYMENT_SYSTEM = "paymentSystem";
	public static final String PAYMENT_CONFIG_GROUP = "paymentConfigGroup";
	
	@Override
	public void performExecute() throws ECException {
		final String METHOD_NAME = "performExecute()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		//Get request properties from ApplePay plugin
		Map properties = getPluginProperties();
		
		//compose request body as JSON
		String requestBody = composeRequestJson(properties);
		
		//execute https request to ApplePay
		String contentType = "application/json; charset=\"utf-8\"";
		String response = ApplePayOrderUtils.executeHttpsRequest(this.validationURL, "POST", requestBody, contentType);
		
		//extract merchant session identifier from response
		this.session = response;
		
		//Put session object to response
		responseProperties = new TypedProperty();
		if(this.session != null){
			responseProperties.put(MERCHANT_SESSION, session);
		}

		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}
	
	@Override
	public void setRequestProperties(com.ibm.commerce.datatype.TypedProperty reqProperties) throws com.ibm.commerce.exception.ECException {
		final String METHOD_NAME = "setRequestProperties";
		Object[] param = {reqProperties};
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, param);
		
		super.setRequestProperties(reqProperties);
		
		this.validationURL = reqProperties.getString(VALIDATION_URL, null);
		this.paymentSystem = reqProperties.getString(PAYMENT_SYSTEM, APPLE_PAY_PLUGIN_NAME);
		this.paymentConfigGroup = reqProperties.getString(PAYMENT_CONFIG_GROUP, "default");
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}

	@Override
	public void validateParameters() throws ECException {
		final String METHOD_NAME = "validateParameters()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);

		if(this.getValidationURL() == null){
            throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME,
            		METHOD_NAME, ECMessageHelper.generateMsgParms("request_url"));			
		}		
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}

	@Override
	public void setValidationURL(String url) {
		this.validationURL = url;
	}

	@Override
	public String getValidationURL() {
		return this.validationURL;
	}

	@Override
	public String getMerchantSession() {
		return this.session;
	}
	
	
	/**
	 * Get Plugin properties for Apple Pay.
	 * @return map
	 * @throws ECException
	 */
	protected Map getPluginProperties() throws ECException{
		final String METHOD_NAME = "getPluginProperties()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);

		//Get ApplePay plugin configuration properties
		Integer storeId = this.getCommandContext().getStoreId();
		TypedProperty pluginProperties = ApplePayOrderUtils.getMerchantInfo(storeId, paymentSystem, paymentConfigGroup);
		
		String merchantIdentifier = null;
		String domainName = null;
		String displayName = null;
		
		if(pluginProperties != null){
			merchantIdentifier = (String) pluginProperties.get(MERCHANT_IDENTIFIER);
			domainName = (String) pluginProperties.get(DOMAIN_NAME);
			displayName = (String) pluginProperties.get(DISPLAY_NAME);
		}
		
		if(merchantIdentifier == null){
            throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME,
            		METHOD_NAME, ECMessageHelper.generateMsgParms(MERCHANT_IDENTIFIER));
		}
		
		if(domainName == null){
            throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME,
            		METHOD_NAME, ECMessageHelper.generateMsgParms(DOMAIN_NAME));
		}
		
		if(displayName == null){
            throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME,
            		METHOD_NAME, ECMessageHelper.generateMsgParms(DISPLAY_NAME));
		}
		
		Map properties = new HashMap();
		properties.put(MERCHANT_IDENTIFIER, merchantIdentifier);
		properties.put(DOMAIN_NAME, domainName);
		properties.put(DISPLAY_NAME, displayName);
		
		Object[] params = {merchantIdentifier, domainName, displayName};
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, params);
		
		return properties;
	}
	
	/**
	 * Compose merchant validation request JSON
	 * @param request properties
	 * @return json String
	 */
	protected String composeRequestJson(Map pluginProperties){
		String jsonStr = null;
		
		JSONObject json = new JSONObject(pluginProperties);
		jsonStr = json.toString();
		
		return jsonStr;
	}
	
	/**
	 * Extract merchant session identifier from response.
	 * @param responseStr
	 * @return session identifier.
	 */
	protected String extractMerchantSessionStr(String responseStr){
		final String METHOD_NAME = "extractMerchantSessionStr()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		String sessionStr = null;
		
		try {
			JSONObject json = new JSONObject(responseStr);
			sessionStr = json.getString(MERCHANT_SESSION_IDENTIFIER).toString();
		} catch (JSONException e) {
			ECTrace.trace(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, "Exception occurred when calling the price service:  "
					+ e.getMessage());
		}
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		return sessionStr;
	}

}
