package com.ibm.commerce.applepay.payment.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;

import com.ibm.commerce.applepay.payment.objects.PKPaymentData;
import com.ibm.commerce.applepay.payment.objects.PKPaymentDataHeader;
import com.ibm.commerce.applepay.payment.objects.PKPaymentMethod;
import com.ibm.commerce.applepay.payment.objects.PKPaymentToken;
import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandContextImpl;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.component.contextservice.ActivityToken;
import com.ibm.commerce.context.base.BaseContext;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.foundation.server.services.businesscontext.ContextService;
import com.ibm.commerce.foundation.server.services.businesscontext.ContextServiceFactory;
import com.ibm.commerce.order.facade.client.OrderException;
import com.ibm.commerce.order.facade.client.OrderFacadeClient;
import com.ibm.commerce.order.facade.datatypes.OrderType;
import com.ibm.commerce.order.facade.datatypes.ShowOrderDataAreaType;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;

public class ApplePayOrderUtils {
	
	/** The IBM Copyright notice field. */
    public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

    /** The name of this class for trace/logging/debuging purposes. */
    private static final String CLASS_NAME = "com.ibm.commerce.applepay.payment.commands.ApplePayOrderUtils";
    
    public static final String UNDEFINED = "undefined";
    
    private static final Long TRACE_COMPONENT = ECTraceIdentifiers.COMPONENT_PPC;
    
	/**
	 * Get the merchant information for ApplePay.
	 * @param storeId
	 * @param paymentSystem
	 * @param paymentConfigGroup
	 * @return merchantInfo
	 * @throws ECException 
	 */
	@SuppressWarnings("deprecation")
	public static TypedProperty getMerchantInfo(Integer storeId, String paymentSystem, String paymentConfigGroup) throws ECException{
		final String METHOD_NAME = "getMerchantInfo";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		TypedProperty merchantInfo = new TypedProperty();
		try{
			GetMerchantInfoCmd getMerchantInfoCmd = (GetMerchantInfoCmd) CommandFactory.createCommand(GetMerchantInfoCmd.NAME, storeId);
			ContextService bcs = ContextServiceFactory.getContextService();
			BaseContext baseCtx = (BaseContext) bcs.findContext(BaseContext.CONTEXT_NAME);
			ActivityToken acToken = baseCtx.getActivityToken();
			CommandContext cmdCtx = new CommandContextImpl(acToken);
			cmdCtx.setStoreId(storeId);
			cmdCtx.setCommand(getMerchantInfoCmd);
			
			TypedProperty requestProperties = new TypedProperty();
			requestProperties.put("storeId", storeId.toString());
			requestProperties.put("paymentSystem", paymentSystem);
			requestProperties.put("paymentConfigGroup", paymentConfigGroup);
			
			getMerchantInfoCmd.setCommandContext(cmdCtx);
			getMerchantInfoCmd.setRequestProperties(requestProperties);
			getMerchantInfoCmd.execute();
			
			merchantInfo = getMerchantInfoCmd.getResponseProperties();
		} catch (Exception e) {
			throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, METHOD_NAME, new Object[] { e.toString() }, e);
		}

		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		return merchantInfo;
	}
	
	/**
	 * Get order noun base on the input orderId.
	 * @param orderId
	 * @param isSummary
	 * @return orderNoun
	 * @throws ECException 
	 */
	public static OrderType getOrderNoun(String orderId, boolean isSummary) throws ECException {
		final String METHOD_NAME = "getOrderNoun";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		OrderType orderNoun = null;
		try{
			String[] orderIdArray = new String[]{orderId};
			OrderFacadeClient orderClient = new OrderFacadeClient();
			ShowOrderDataAreaType showOrder = orderClient.findByOrderId(orderIdArray, isSummary);
			List orderNouns = showOrder.getOrder();
			if(orderNouns != null && !orderNouns.isEmpty()){
				orderNoun = (OrderType) orderNouns.get(0);
			}
		}catch(OrderException e){
			throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, METHOD_NAME, new Object[] { e.toString() }, e);
		}
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		return orderNoun;
	}
	
	/**
	 * Compose PKPaymentToken base on PI's extended data.
	 * @param extData
	 * @return PKPaymentToken
	 */
	public static PKPaymentToken composePaymentToken(Map extData){
		final String METHOD_NAME = "composePaymentToken";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		//compose PKPaymentMethod
		PKPaymentMethod paymentMethod = new PKPaymentMethod();
		String displayName = (String) extData.get("paymentMethod_displayName");
		String network = (String) extData.get("paymentMethod_network");
		String type = (String) extData.get("paymentMethod_type");
		String paymentPass = (String) extData.get("paymentMethod_paymentPass");
		if(displayName != null && displayName != UNDEFINED){
			paymentMethod.setDisplayName(displayName);
		}
		if(network != null){
			paymentMethod.setNetwork(network);
		}
		if(type != null){
			paymentMethod.setType(type);
		}
		if(paymentPass != null){
			paymentMethod.setPaymentPass(paymentPass);
		}
		
		//compose PKPaymentDataHeader
		PKPaymentDataHeader header = new PKPaymentDataHeader();
		String applicationData = (String) extData.get("paymentData_header_applicationData");
		String ephemeralPublicKey = (String) extData.get("paymentData_header_ephemeralPublicKey");
		String wrappedKey = (String) extData.get("paymentData_header_wrappedKey");
		String publicKeyHash = (String) extData.get("paymentData_header_publicKeyHash");
		String transactionId = (String) extData.get("paymentData_header_transactionId");
		if(applicationData != null && applicationData != UNDEFINED){
			header.setApplicationData(applicationData);
		}
		if(ephemeralPublicKey != null && ephemeralPublicKey != UNDEFINED){
			header.setEphemeralPublicKey(ephemeralPublicKey);
		}
		if(wrappedKey != null && wrappedKey != UNDEFINED){
			header.setWrappedKey(wrappedKey);
		}
		if(publicKeyHash != null && publicKeyHash != UNDEFINED){
			header.setPublicKeyHash(publicKeyHash);
		}
		if(transactionId != null && transactionId != UNDEFINED){
			header.setTransactionId(transactionId);
		}
		
		//compose PKPaymentData
		PKPaymentData paymentData = new PKPaymentData();
		String data = (String) extData.get("paymentData_data");
		String signature = (String) extData.get("paymentData_signature");
		String version = (String) extData.get("paymentData_version");
		if(data != null && data != UNDEFINED){
			paymentData.setData(data);
		}
		if(signature != null && signature != UNDEFINED){
			paymentData.setSignature(signature);
		}
		if(version != null && version != UNDEFINED){
			paymentData.setVersion(version);
		}
		if(header != null){
			paymentData.setHeader(header);
		}
		
		
		//compose PKPaymentToken
		PKPaymentToken paymentToken = new PKPaymentToken();
		String transactionIdentifier = (String) extData.get("transactionIdentifier");
		if(paymentData != null){
			paymentToken.setpaymentData(paymentData);
		}
		if(paymentMethod != null){
			paymentToken.setPaymentMethod(paymentMethod);
		}
		if(transactionIdentifier != null){
			paymentToken.setTransactionIdentifier(transactionIdentifier);
		}
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		return paymentToken;
	}
	
	/**
	 * Execute the HTTPS request to ApplePay merchant validation service.
	 * @param requestURL
	 * @param requestMethod
	 * @param requestBody
	 * @param contentType
	 * @return response
	 * @throws ECException
	 */
	public static String executeHttpsRequest(String requestURL, String requestMethod, String requestBody, String contentType) throws ECException{
		final String METHOD_NAME = "executeHttpsRequest()";
		String result = "";

		int connectTimeout = 120000;
		int connectInterval = 5;
		int readTimeout = 120000;
		HttpsURLConnection conn = null;
        BufferedWriter bw = null;
        BufferedReader br = null;

		try {
			URI uri = new URI(requestURL);
			URL url = uri.toURL();
//			URL url = new URL(requestURL);
			conn = (HttpsURLConnection) url.openConnection();
			conn.setDoInput(true);
			conn.setDoOutput(true);
	        conn.setConnectTimeout(connectTimeout);
	        conn.setReadTimeout(readTimeout);
	        conn.setRequestMethod(requestMethod.toUpperCase());
	        conn.setRequestProperty("Content-Type", contentType);
	        conn.setHostnameVerifier(new HostnameVerifierImpl());
	        //log
	        conn.connect();
	        //log
	        
	        //Send request
	        InputStream is = null;
	        try{
	        	if(requestBody != null){
	        		bw = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream(),"UTF-8"));
	        		bw.write(requestBody);
	        		bw.flush();
	        	}
	        } catch (IOException e) {
	        	throw new ECApplicationException(ECMessage._ERR_IO_EXCEPTION, CLASS_NAME,
	            		METHOD_NAME);
			}finally{
				if(bw != null){
					bw.close();
				}
			}
	        
	        //Get response
	        try{
	        	br = new BufferedReader(new InputStreamReader(conn.getInputStream()));
	        	StringBuffer sb = new StringBuffer();
	        	String line = br.readLine();
	        	if(line != null){
	        		sb.append(line);
	        		line = br.readLine();
	        	}
	        	result = sb.toString();
	        }catch(IOException e){
				throw new ECApplicationException(ECMessage._ERR_IO_EXCEPTION, CLASS_NAME,
	            		METHOD_NAME);
	        }finally{
	        	if(br != null){
	        		br.close();
	        	}
	        }
		} catch (IOException e) {
			
			throw new ECApplicationException(ECMessage._ERR_IO_EXCEPTION, CLASS_NAME,
            		METHOD_NAME);
		} catch (URISyntaxException e) {
			throw new ECApplicationException(ECMessage._ERR_GENERIC, CLASS_NAME,
            		METHOD_NAME);
		}finally{
			try{
				if(bw != null){
					bw.close();
				}
				if(br != null){
					br.close();
				}
			}catch(IOException ioe){
				ioe.printStackTrace();
			}

		}
		return result;
	}
	
	private static class HostnameVerifierImpl implements HostnameVerifier{
		@Override
		public boolean verify(String hostname, SSLSession session) {
			return true;
		}		
	}

}
