package com.ibm.commerce.applepay.payment.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.Map;

import com.ibm.commerce.applepay.payment.objects.PKPaymentToken;
import com.ibm.commerce.command.TaskCommandImpl;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.edp.utils.Constants;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.order.facade.datatypes.OrderType;
import com.ibm.commerce.payments.plugin.FinancialTransaction;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECMessageHelper;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;

/**
 * This class is the default implementation of <code>ProcessApplePayPaymentCmd</code>.
 * It is invoked by ApplePayBean to process the payment token from ApplePay.
 * You can customize the corresponding methods to compose necessary information including the 
 * Payment token to the real Payment Gateway for processing, then set back the result to object:
 * <code>financialTransaction</code> which will be passed back to WCS PPC framework.
 * 
 * Here's the example for how to set this object:
 * financialTransaction.setState(FinancialTransaction.STATE_SUCCESS);
 * financialTransaction.setReasonCode(PluginConstants.RESPONSECODE_SUCCESS);
 * financialTransaction.setProcessedAmount(processedAmount);
 * etc.
 * 
 * In current version, only approve(), deposit() and approveAndDeposit() methods are considered,
 * additional methods will be provided later.
 */
public class ProcessApplePayPaymentCmdImpl extends TaskCommandImpl implements
		ProcessApplePayPaymentCmd {
	
	public static final String CLASS_NAME = ProcessApplePayPaymentCmdImpl.class.getName();
	private static final Long TRACE_COMPONENT = ECTraceIdentifiers.COMPONENT_PPC;
	
	private PKPaymentToken paymentToken;
	private String paymentAction;
	private String paymentResult;
	private TypedProperty merchantInfo;
	private FinancialTransaction financialTransaction;
	private OrderType orderNoun;
	private TypedProperty extParams;

	@Override
	public void setPaymentToken(PKPaymentToken paymentToken) {
		this.paymentToken = paymentToken;
	}
	
	/**
	 * Get the payment token
	 * @return paymentToken
	 */
	public PKPaymentToken getPaymentToken(){
		return this.paymentToken;
	}

	/**
	 * Set the payment action, eg. Approve, Deposit etc.
	 * @param action
	 */
	public void setPaymentAction(String action){
		this.paymentAction = action;
	}
	
	/**
	 * Get the payment action, eg. Approve, Deposit etc. 
	 * @return action
	 */
	public String getPaymentAction(){
		return this.paymentAction;
	}
	
	/**
	 * Set merchant information.
	 * @param merchantInfo
	 */
	public void setMerchantInfo(TypedProperty merchantInfo){
		this.merchantInfo = merchantInfo;
	}
	
	/**
	 * Get merchant information.
	 * @return financialTransaction
	 */
	public Map getMerchantInfo(){
		return this.merchantInfo;
	}
	
	/**
	 * Set the FinancialTransaction for the given payment action.
	 * @param financialTransaction
	 */
	public void setFinancialTransaction(FinancialTransaction financialTransaction){
		this.financialTransaction = financialTransaction;
	}
	
	/**
	 * Get the FinancialTransaction result.
	 * @return financialTransaction
	 */
	public FinancialTransaction getFinancialTransaction(){
		return this.financialTransaction;
	}
	
	/**
	 * Set the order noun object.
	 * @param orderNoun
	 */
	public void setOrderNoun(OrderType orderNoun){
		this.orderNoun = orderNoun;
	}
	
	/**
	 * Get the order noun object.
	 * @return orderNoun
	 */
	public OrderType getOrderNoun(){
		return this.orderNoun;
	}
	
	/**
	 * Set the extended parameters.
	 * @param extParams
	 */
	public void setExtendedParameters(TypedProperty extParams){
		this.extParams = extParams;
	}
	
	/**
	 * Get the extended parameters.
	 * @return extParams
	 */
	public TypedProperty getExtendedParameters(){
		return this.extParams;
	}
	
	@Override
	public void validateParameters() throws ECException {
		final String METHOD_NAME = "validateParameters()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);

		if(this.getPaymentAction() == null){
            throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME,
            		METHOD_NAME, ECMessageHelper.generateMsgParms("paymentAction"));			
		}
		
		if(this.getPaymentToken() == null){
            throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME,
            		METHOD_NAME, ECMessageHelper.generateMsgParms("paymentToken"));			
		}
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}	
	
	/**
	 * The common steps for the business logic:
	 * 1. Compose the necessary request properties including the payment token
	 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
	 * 3. Analyze payment result from Payment Gateway	
	 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
	 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
	 */
	@Override
	public void performExecute() throws ECException {
		final String METHOD_NAME = "performExecute()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		if(this.paymentAction.equalsIgnoreCase(Constants.EDP_BACKEND_ACTION_APPROVE)){
			approve();
		}else if(this.paymentAction.equalsIgnoreCase(Constants.EDP_BACKEND_ACTION_DEPOSIT)){
			deposit();
		}else if(this.paymentAction.equalsIgnoreCase(Constants.EDP_BACKEND_ACTION_APPROVE_AND_DEPOSIT)){
			approveAndDeposit();
		}
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}
	
	/**
	 * This method is used for executing the Approve action.
	 */
	protected void approve(){
		String result = null;
		final String METHOD_NAME = "approve()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}
	
	/**
	 * This method is used for executing the Deposit action.
	 */
	protected void deposit(){
		String result = null;
		final String METHOD_NAME = "deposit()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */

		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}

	/**
	 * This method is used for executing the Deposit action.
	 */
	protected void approveAndDeposit(){
		String result = null;
		final String METHOD_NAME = "approveAndDeposit()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */

		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}

	

}
