package com.ibm.commerce.applepay.payment.commands;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.util.HashMap;

import com.ibm.commerce.applepay.payment.objects.PKPaymentToken;
import com.ibm.commerce.payments.plugin.FinancialTransaction;
import com.ibm.commerce.payments.plugin.PluginException;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;

public class ProcessApplePayPaymentSimulatorCmdImpl extends
		ProcessApplePayPaymentCmdImpl implements ProcessApplePayPaymentCmd {
	
	public static final String CLASS_NAME = ProcessApplePayPaymentSimulatorCmdImpl.class.getName();
	private static final Long TRACE_COMPONENT = ECTraceIdentifiers.COMPONENT_PPC;
	public static final String SIMULATOR_IP = "9.111.139.100";
	public static final int SIMULATOR_PORT = 5656;
	public static final int RESPONSE_TIMEOUT = 5000;
	
	public static final String PROCESSED_AMOUNT_KEY = "processedAmount";
	public static final String RETURN_STATUS_KEY = "returnStatus";
	public static final String RETURN_STATUS_VALUE_SUCCESS = "success";
	public static final String RETURN_STATUS_VALUE_FAIL = "fail";
	
	/**
	 * This method is used for executing the Approve action.
	 */
	@Override
	protected void approve(){
		final String METHOD_NAME = "approve()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */
		
		PKPaymentToken token = this.getPaymentToken();
		FinancialTransaction financialTransaction = this.getFinancialTransaction();
		handlePaymentRequest(token, financialTransaction, METHOD_NAME);
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}
	
	/**
	 * This method is used for executing the Deposit action.
	 */
	@Override
	protected void deposit(){
		String result = null;
		final String METHOD_NAME = "deposit()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */
		
		PKPaymentToken token = this.getPaymentToken();
		FinancialTransaction financialTransaction = this.getFinancialTransaction();
		handlePaymentRequest(token, financialTransaction, METHOD_NAME);

		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}
	
	/**
	 * This method is used for executing the Deposit action.
	 */
	protected void approveAndDeposit(){
		String result = null;
		final String METHOD_NAME = "approveAndDeposit()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */

		PKPaymentToken token = this.getPaymentToken();
		FinancialTransaction financialTransaction = this.getFinancialTransaction();
		handlePaymentRequest(token, financialTransaction, METHOD_NAME);
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}	
	
	private void handlePaymentRequest(PKPaymentToken token, FinancialTransaction financialTransaction, String methodName){
		final String METHOD_NAME = methodName;
		String ephemeralPubKey = token.getpaymentData().getHeader().getEphemeralPublicKey();
		String data = token.getpaymentData().getData();
		String signature = token.getpaymentData().getSignature();
		System.out.println("ephemeralPubKey is:>"+ephemeralPubKey+"<");
		System.out.println("data is:>"+data+"<");
		System.out.println("signature is:>"+signature+"<");

		try{
			//process result from Gateway (Simulator), and set back to financialTransaction
			HashMap<String,String> result = processPaymentRequestBySimulator(token);
			String returnStatus = result.get(RETURN_STATUS_KEY);
			String processedAmount = result.get(PROCESSED_AMOUNT_KEY);
			if(returnStatus != null && processedAmount != null){
				ECTrace.trace(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, "returnStatus is: '" + returnStatus + "'.");
				ECTrace.trace(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, "processedAmount is: '" + processedAmount + "'.");
				BigDecimal processedAmountBD = new BigDecimal(processedAmount);
				
				financialTransaction.setProcessedAmount(processedAmountBD);
				if(returnStatus.equalsIgnoreCase(RETURN_STATUS_VALUE_SUCCESS)){
					financialTransaction.setState(FinancialTransaction.STATE_SUCCESS);
					financialTransaction.setResponseCode("0");
				}else if(returnStatus.equalsIgnoreCase(RETURN_STATUS_VALUE_FAIL)){
					financialTransaction.setState(FinancialTransaction.STATE_FAILED);
				}
			}
		}catch(PluginException e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Send payment request to simulator.
	 * @param token
	 * @return result
	 */
	protected HashMap<String,String> processPaymentRequestBySimulator(PKPaymentToken token){
		final String METHOD_NAME = "processPaymentRequestBySimulator()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		HashMap<String,String> result = new HashMap<String,String>();
		
		Socket socket = new Socket();
		
		try{
			SocketAddress addr = new InetSocketAddress(SIMULATOR_IP,SIMULATOR_PORT);
			socket.connect(addr, 10000);
			
			//pass payment request to simulator
			OutputStream ops = socket.getOutputStream();
			ObjectOutputStream oops = new ObjectOutputStream(ops);
			oops.writeObject(token);
			
			//read response from simulator synchronously
			long startTime = System.currentTimeMillis();
			InputStream is = socket.getInputStream();
			for(;;){
				if(is != null && is.available() > 0){
					ObjectInputStream oips = new ObjectInputStream(is);
					result = (HashMap<String, String>) oips.readObject();
					break;
				}
				
				long currentTime = System.currentTimeMillis();
				if(currentTime - startTime > RESPONSE_TIMEOUT){
					ECTrace.trace(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, "Exceed Max wait time for reading response, terminate the connection.");
					break;
				}
			}
			
		}catch(IOException e){
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}finally{
			try {
				socket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		return result;
	}

}
