package com.ibm.commerce.applepay.rest.handler;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.Map;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.ws.rs.Encoded;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import org.apache.wink.common.http.HttpStatus;

import com.ibm.commerce.applepay.order.commands.ApplePayOrderCancelCmd;
import com.ibm.commerce.applepay.order.commands.ApplePayOrderPrepareCmd;
import com.ibm.commerce.applepay.order.commands.ApplePayOrderProcessCmd;
import com.ibm.commerce.applepay.order.commands.ApplePayOrderUpdateCmd;
import com.ibm.commerce.applepay.payment.commands.ApplePayMerchantValidationCmd;
import com.ibm.commerce.applepay.payment.commands.GetMerchantInfoCmd;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.foundation.common.util.logging.LoggingHelper;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.rest.bod.helpers.OrderHelper;
import com.ibm.commerce.rest.classic.core.AbstractConfigBasedClassicHandler;
import com.ibm.commerce.rest.javadoc.AdditionalParameterList;
import com.ibm.commerce.rest.javadoc.Description;
import com.ibm.commerce.rest.javadoc.InternalProduces;
import com.ibm.commerce.rest.javadoc.ParameterDescription;
import com.ibm.commerce.rest.javadoc.ResponseCode;
import com.ibm.commerce.rest.javadoc.ResponseSchema;
import com.ibm.commerce.rest.javadoc.ResponseType;
import com.ibm.commerce.rest.javadoc.StoreIdProvider;

/**
 * Apple pay resource.
 */
@Path("store/{storeId}/applepay")
@Encoded
public class ApplePayHandler extends AbstractConfigBasedClassicHandler {

	private static final String CLASSNAME = ApplePayHandler.class.getName();
	private static final Logger LOGGER = LoggingHelper
			.getLogger(ApplePayHandler.class);

	private static final String RESOURCE_NAME = "applepay";
	
	private static final String DEFAULT_IBM_SUMMARY_PROFILE_NAME = "IBM_Summary";
	private static final String PROFILE_NAME_MY_COMMAND_SUMMARY = "MyCommand_Summary";
	
	private static final String PARAMETER_ORDER_ID = "orderId";
	private static final String PARAMETER_ORDER_ID_DESCRIPTION = "The order ID.";
	
	/** The Constant RESOURCE_MAPPING_KEY. */
	public static final String ORDER_SHIPPING_UPDATE_MAPPING_KEY = "shippingInfoUpdate";
	
	/**
	 * The merchant validation URL.
	 */
	public static final String PARAMETER_VALIDATION_URL = "validationURL";
	/**
	 * The merchant validation URL description.
	 */
	public static final String PARAMETER_VALIDATION_URL_DESCRIPTION = "merchant validation URL";

	
	public static final String PAYMENT_SYSTEM = "paymentSystem";
	public static final String PAYMENT_CONFIG_GROUP = "paymentConfigGroup";
	public static final String PAYMENT_SYSTEM_DESCRIPTION = "The payment system which is used for ApplePay";
	public static final String PAYMENT_CONFIG_GROUP_DESCRIPTION = "The payment configuration group";
	
	/**
	 * Helper to retrieve and set data in BOD
	 */
	private OrderHelper helper = new OrderHelper(getInstrumentor());
	
	/**
	 * store id.
	 */
	@PathParam("storeId") 
	String storeId;
	

	/**
	 * Get store id.
	 */
	public String getStoreId() {
		return storeId;
	}

	/**
	 * Set store id.
	 * @param storeId
	 */
	public void setStoreId(String storeId) {
		this.storeId = storeId;
	}

	/**
	 * Get resource name.
	 */
	public String getResourceName() {
		return RESOURCE_NAME;
	}

	
	/**
	 * This method performs the merchant validation.
	 * 
	 * @param storeId the store Id.
	 * @param responseFormat the response format (xml, json, or atom).
	 * @return A map of String [] that contains the following response parameters: merchantSession
	 */		
	@POST
	@Path("merchant/validate")
	@Produces({ APPLICATION_JSON, APPLICATION_XML, APPLICATION_XHTML_XML,
		APPLICATION_ATOM_XML})
	@Description("Performs the merchant validation")
	@ResponseSchema(valueClass = String.class, responseCodes = {
		@ResponseCode(code = 201, reason = RESPONSE_201_DESCRIPTION),
		@ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
		@ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
		@ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
		@ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION) })
	public Response performMerchantValidation(
			@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@QueryParam(value = PARAMETER_RESPONSE_FORMAT) @ParameterDescription(description = PARAMETER_RESPONSE_FORMAT_DESCRIPTION, valueProviderClass = ResponseType.class) String responseFormat) { 
	    
		final String METHODNAME = "performMerchantValidation(String storeId, String validationURL, String responseFormat)";
		      
		String validationURL = request.getParameter(PARAMETER_VALIDATION_URL);
		String paymentSystem = request.getParameter(PAYMENT_SYSTEM);
		String paymentConfigGroup = request.getParameter(PAYMENT_CONFIG_GROUP);

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			Object[] objArr = new Object[] { storeId, validationURL, responseFormat };
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}
	    	  	
	    Response result = null;
	    String test=null;
     	try {
    		Response response = prepareAndValidate(storeId, getResourceName(), POST_METHOD, request, responseFormat);
    		if (response == null){
    			try {
    				String cmdRefKey = ApplePayMerchantValidationCmd.NAME;
    				TypedProperty requestProperties = new TypedProperty();
   					requestProperties.put(PARAMETER_VALIDATION_URL, validationURL);
   					requestProperties.put(PAYMENT_SYSTEM, paymentSystem);
   					requestProperties.put(PAYMENT_CONFIG_GROUP, paymentConfigGroup);
   					
   					result = executeControllerCommandWithContext(storeId, cmdRefKey, requestProperties, responseFormat);
    			} catch (Exception ex) {
    				// Call handleException to generate response with error status code
    				result = handleException(responseFormat, ex, METHODNAME);
    			}
    		}
	    	
	    } catch (Exception ex) {
    		// Call handleException to generate response with error status code
    		result = handleException(responseFormat, ex, METHODNAME);
	    }
        	
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

	    return result;

	}	
	
	/**
	 * Prepare the apple pay cart.
	 * 
	 * @param storeId
	 * @param responseFormat
	 * @return
	 * @throws Exception
	 */
	@POST
	@Produces({ APPLICATION_JSON, APPLICATION_XML, APPLICATION_XHTML_XML,
		APPLICATION_ATOM_XML})
	@Path("cart/prepare")
	@Description("Prepare the apple pay cart.")
	@AdditionalParameterList({
		@ParameterDescription(name = ParameterDescription.BODY_NAME, paramType = ParameterDescription.BODY_TYPE, description = "The request object for ApplePayPrepare.", typeClass = ApplePayPrepareBodyDescription.class) })

	@ResponseSchema(responseCodes = {
		@ResponseCode(code = 200, reason = RESPONSE_200_DESCRIPTION),
		@ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
		@ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
		@ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
		@ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION)
	})
	
	public Response applePayPrepare(
			@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@QueryParam(value = PARAMETER_RESPONSE_FORMAT) @ParameterDescription(description = PARAMETER_RESPONSE_FORMAT_DESCRIPTION, valueProviderClass = ResponseType.class) String responseFormat
			) throws Exception{


		final String METHODNAME = "applePayPrepare(String storeId, String responseFormat)";
		boolean entryExitTraceEnabled = LoggingHelper.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			Object[] objArr = new Object[] {storeId, responseFormat};
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}
		Response response = prepareAndValidate(storeId, getResourceName(), POST_METHOD, request, responseFormat);
		if (response == null){
			String cmdRefKey = ApplePayOrderPrepareCmd.NAME;
			TypedProperty requestProperties = null;
			requestProperties = initializeRequestPropertiesFromRequestMap(responseFormat);
			
			response = executeControllerCommandWithContext(storeId,cmdRefKey,requestProperties,responseFormat);
		}
			
		if (entryExitTraceEnabled) {
			LOGGER.exiting(CLASSNAME, METHODNAME, response);
		}

		return response;
	
	}
	
	
	/**
	 * Process the apple pay cart.
	 * 
	 * @param storeId
	 * @param responseFormat
	 * @return
	 * @throws Exception
	 */
	@POST
	@Produces({ APPLICATION_JSON, APPLICATION_XML, APPLICATION_XHTML_XML,
		APPLICATION_ATOM_XML})
	@Path("cart/process")
	@Description("Process the apple pay order.")
	@AdditionalParameterList({
		@ParameterDescription(name = ParameterDescription.BODY_NAME, paramType = ParameterDescription.BODY_TYPE, description = "The request object for ApplePayProcess.", typeClass = ApplePayProcessBodyDescription.class) })

	@ResponseSchema(responseCodes = {
		@ResponseCode(code = 200, reason = RESPONSE_200_DESCRIPTION),
		@ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
		@ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
		@ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
		@ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION)
	})
	public Response applePayProcess(
			@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@QueryParam(value = PARAMETER_RESPONSE_FORMAT) @ParameterDescription(description = PARAMETER_RESPONSE_FORMAT_DESCRIPTION, valueProviderClass = ResponseType.class) String responseFormat
			) throws Exception{
		
		final String METHODNAME = "applePayProcess(String storeId, String responseFormat)";
		boolean entryExitTraceEnabled = LoggingHelper.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			Object[] objArr = new Object[] {storeId, responseFormat};
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}
		Response response = prepareAndValidate(storeId, getResourceName(), POST_METHOD, request, responseFormat);

		if (response == null){
			String cmdRefKey = ApplePayOrderProcessCmd.NAME;
			TypedProperty requestProperties = null;
			requestProperties = initializeRequestPropertiesFromRequestMap(responseFormat);
			response = executeControllerCommandWithContext(storeId,cmdRefKey,requestProperties,responseFormat);
		}
			
		if (entryExitTraceEnabled) {
			LOGGER.exiting(CLASSNAME, METHODNAME, response);
		}

		return response;
		
	}
	
	/**
	 * Update the apple pay order.
	 * 
	 * @param storeId
	 * @param responseFormat
	 * @return
	 * @throws Exception
	 */
	@PUT
	@Path("cart")
	@Produces({ APPLICATION_JSON, APPLICATION_XML, APPLICATION_XHTML_XML,
		APPLICATION_ATOM_XML})
	
	@Description("Update the apple pay order.")
	@AdditionalParameterList({
		@ParameterDescription(name = ParameterDescription.BODY_NAME, paramType = ParameterDescription.BODY_TYPE, description = "The request object for Apple pay order update.", typeClass = ApplePayUpdateBodyDescription.class) })

	@ResponseSchema(responseCodes = {
		@ResponseCode(code = 200, reason = RESPONSE_200_DESCRIPTION),
		@ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
		@ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
		@ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
		@ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION)
	})
	public Response updateApplePayOrder(
			@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@QueryParam(value = PARAMETER_RESPONSE_FORMAT) @ParameterDescription(description = PARAMETER_RESPONSE_FORMAT_DESCRIPTION, valueProviderClass = ResponseType.class) String responseFormat
			) throws Exception{
		final String METHODNAME = "updateApplePayOrder(String storeId, String responseFormat)";
		
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			Object[] objArr = new Object[] { storeId, responseFormat };
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}
	    
		Response response = prepareAndValidate(storeId, getResourceName(), POST_METHOD, request, responseFormat);
		if (response == null){
			String cmdRefKey = ApplePayOrderUpdateCmd.NAME;
			
			TypedProperty requestProperties = initializeRequestPropertiesFromRequestMap(responseFormat);		
			response = executeControllerCommandWithContext(storeId,cmdRefKey,requestProperties,responseFormat);
		} 	
    	
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			Object[] objArr = new Object[] { storeId, responseFormat };
			LOGGER.exiting(CLASSNAME, METHODNAME, objArr);
		}

    	return response;
    	
	}
	
	/**
	 * Render response data for processing or changing data using a Map instead
	 * of DataObject.
	 * 
	 * @param resultData
	 *            a Map
	 * @param responseFormat
	 *            response format
	 * @param status HTTP status
	 * @return Response
	 */
	protected Response renderUpdateShippingInfo(
			Map<String, Object> resultData, String responseFormat, HttpStatus status) {
		final String METHODNAME = "renderUpdateShippingInfo";

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME);
		}

		Map<String, Object> dataMap = createMapForProviderWithResultData(
				resultData, getResourceName());

		Response result = generateResponseFromHttpStatusCodeAndRespData(
				responseFormat, dataMap, status);

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

		return result;
	}

	
	
	@POST
	@Produces({ APPLICATION_JSON, APPLICATION_XML, APPLICATION_XHTML_XML,
		APPLICATION_ATOM_XML})
	@Path("cart/cancel")
	@Description("Cancel the apple pay order.")
	@ResponseSchema(responseCodes = {
		@ResponseCode(code = 200, reason = RESPONSE_200_DESCRIPTION),
		@ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
		@ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
		@ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
		@ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION)
	})
	public Response applePayCancel(
			@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@QueryParam(value = PARAMETER_RESPONSE_FORMAT) @ParameterDescription(description = PARAMETER_RESPONSE_FORMAT_DESCRIPTION, valueProviderClass = ResponseType.class) String responseFormat
			) throws Exception{
		
		final String METHODNAME = "applePayCancel(String storeId, String responseFormat)";
		boolean entryExitTraceEnabled = LoggingHelper.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			Object[] objArr = new Object[] {storeId, responseFormat};
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}
		Response response = prepareAndValidate(storeId, getResourceName(), POST_METHOD, request, responseFormat);
		if (response == null){
			String cmdRefKey = ApplePayOrderCancelCmd.NAME;
			TypedProperty requestProperties = null;
			requestProperties = initializeRequestPropertiesFromRequestMap(responseFormat);
			response = executeControllerCommandWithContext(storeId,cmdRefKey,requestProperties,responseFormat);
		}
			
		if (entryExitTraceEnabled) {
			LOGGER.exiting(CLASSNAME, METHODNAME, response);
		}

		return response;
		
	}

	
	/**
	 * This method gets the merchant info.
	 * 
	 * @param storeId the store Id.
	 * @param paymentSystem the payment system name
	 * @param paymentConfigGroup the payment configuration group name
	 * @param responseFormat the response format (xml, json, or atom).
	 * @return A map of String [] that contains the following response parameters: merchantSession
	 */		
	@GET
	@Path("merchant")
	@InternalProduces({ APPLICATION_JSON, APPLICATION_XML,APPLICATION_XHTML_XML,
		APPLICATION_ATOM_XML })
	@Description("Gets the merchant info")
	@ResponseSchema(valueClass = String.class, responseCodes = {
		@ResponseCode(code = 201, reason = RESPONSE_201_DESCRIPTION),
		@ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
		@ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
		@ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
		@ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION) })
	public Response getMerchantInfo(
			@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@QueryParam(PAYMENT_SYSTEM) @ParameterDescription(description = PAYMENT_SYSTEM_DESCRIPTION, required = true) String paymentSystem,
			@QueryParam(PAYMENT_CONFIG_GROUP) @ParameterDescription(description = PAYMENT_CONFIG_GROUP_DESCRIPTION, required = true) String paymentConfigGroup,
			@QueryParam(value = PARAMETER_RESPONSE_FORMAT) @ParameterDescription(description = PARAMETER_RESPONSE_FORMAT_DESCRIPTION, valueProviderClass = ResponseType.class) String responseFormat) { 
	    
		final String METHODNAME = "getMerchantInfo(String storeId, String paymentSystem, String paymentConfigGroup, String responseFormat)";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			Object[] objArr = new Object[] { storeId, paymentSystem, paymentConfigGroup, responseFormat };
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}
	    	  	
	    Response result = null;
	    
     	try {
    		Response response = prepareAndValidate(storeId, getResourceName(), POST_METHOD, request, responseFormat);
    		if (response == null){
    			try {
    				String cmdRefKey = GetMerchantInfoCmd.NAME;
    				TypedProperty requestProperties = new TypedProperty();
    				requestProperties.put("storeId", storeId);
    				requestProperties.put("paymentSystem", paymentSystem);
    				requestProperties.put("paymentConfigGroup", paymentConfigGroup);
   					result = executeControllerCommandWithContext(storeId, cmdRefKey, requestProperties, responseFormat);
    			} catch (Exception ex) {
    				// Call handleException to generate response with error status code
    				result = handleException(responseFormat, ex, METHODNAME);
    			}
    		}
	    	
	    } catch (Exception ex) {
    		// Call handleException to generate response with error status code
    		result = handleException(responseFormat, ex, METHODNAME);
	    }
        	
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	    
	    return result;

	}	
	
	
	@Description("Description of the ApplePayPrepare input body.")
	private static class ApplePayPrepareBodyDescription{
		@ParameterDescription(description = "order ID", required = false)
		private String orderId;	
		@ParameterDescription(description = "The flag to indicate adding unbound payment instruction or not.", required = false)
		private boolean addUnboundPI;
	}
	
	@Description("Description of the ApplePayProcess input body.")
	private static class ApplePayProcessBodyDescription {
		//define the body of the ApplePay process
		@ParameterDescription(description = "order ID", required = true)
		private String orderId;
		@ParameterDescription(description = "The unbound payment instruction id for payment type promotion.", required = false)
		private String unboundPIId;
		
		@Description("Description of applePayShippingAddress.")
		private static class ApplePayShippingAddressDescription {
			
			private String country;
			
			private String city;
			
			private String state;
			
			private String zipCode;
			
			private String address1;
			
			private String address2;
			
			private String lastName;
			
			private String firstName;
			
			private String email1;
			
			private String mobilePhone1;
			
		}
		
		
		@Description("Description of applePayBillingAddress.")
		private static class ApplePayBillingAddressDescription {
			
			private String country;
			
			private String city;
			
			private String state;
			
			private String zipCode;
			
			private String address1;
			
			private String address2;
			
			private String lastName;
			
			private String firstName;
					
		}
		
		@ParameterDescription(description = "apple pay shipping contact description.", required = false)
		private ApplePayShippingAddressDescription shippingContact;
		
		@ParameterDescription(description = "applepay billing contact  description.", required = false)
		private ApplePayBillingAddressDescription billingContact;
		
	}
	
	@Description("Description of the ApplePayUpdate input body.")
	private static class ApplePayUpdateBodyDescription{

		
		
		@Description("Description of applePayAddress.")
		private static class ApplePayAddressDescription {
			
			private String country;
			
			private String city;
			
			private String state;
			
			private String zipCode;
			
		}
		
		@ParameterDescription(description = "applePayAddress description.", required = false)
		private ApplePayAddressDescription applePayAddress;
		
		@ParameterDescription(description = "Shipping mode identifier.", required=false)
		private String shipModeId;

		@ParameterDescription(description = "Address identifier.")
		private String addressId;	
		
		@ParameterDescription(description = "The order ID")
		private String orderId;
		
		@ParameterDescription(description = "The identifier for the type of calculation to be performed on the order.")
		private String x_calculationUsage;
		
		private String x_inventoryValidation;
		
		
	}
}
